﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using AIOUSBNet;

namespace ADCContinuous
{
    public partial class Form1 : Form
    {
        public static AIOUSB.ADCallback myCallBack;
        private UInt32 DeviceIndex;
        public UInt16[] ADData = new UInt16[128 * 1024]; //8K scans of 16 channels, or 16K scans of 8 differential channels.
        public delegate void UpdateDataFunc(string toShow);

        public int TotalAmountOfDataReceived { get; private set; }

        public Form1()
        {
            InitializeComponent();
        }

        private void InitButton_Click(object sender, EventArgs e)
        {
            DeviceIndex = AIOUSB.ResolveDeviceIndex(AIOUSB.diOnly);
            if (DeviceIndex == AIOUSB.diNone)
            {
                StatusTextBox.Text += "This sample needs a single USB-AIx device, Revision E or higher\n\r";
                return;
            }
            GoButton.Visible = true;
            StatusTextBox.Text += "One AIOUSB card found\r\n";
        }
     
        private void GoButton_Click(object sender, EventArgs e)
        {
            GoButton.Visible = false;
            StopButton.Visible = true;
            double Hz = 0.0;
            uint configlen = 21;
            byte[] configbuf = new byte[configlen];

            var status = AIOUSB.CTR_StartOutputFreq(DeviceIndex, 0, ref Hz); // stop any running ADC counter
            StatusTextBox.Text += "stopped ADC clock: code " + status.ToString() + "\r\n";

            AIOUSB.ADC_GetConfig(DeviceIndex, configbuf, ref configlen);
            configbuf[0x11] = 0x05; // set timer-start mode
            configbuf[0x12] = 0xF0; // all 16 channels
            AIOUSB.ADC_SetConfig(DeviceIndex, configbuf, ref configlen);

            Hz = 10000.0;
            
            status = AIOUSB.ADC_BulkContinuousCallbackStart(DeviceIndex, 512, 64, 0, myCallBack);
            StatusTextBox.Text += "started: code " + status.ToString() + "\r\n";
            status = AIOUSB.CTR_StartOutputFreq(DeviceIndex, 0, ref Hz); //start ADC at 10kHz
        }

        public void UpdateData(string message)
        {
            DataTextBox.Text = message;
        }

        public void ADCallbackReport(IntPtr pBuf, UInt32 BufSize, UInt32 Flags, UInt32 Context)
        {
            // This flag (2) tells us we've got the last continuous callback packet:
            if (((Flags & 2) != 0))
            {
                String strEnd = "\r\n Stream ended...";  // Be sure this isnt blcoked by stop button func on UI thread
                
            }

            int LastChanInBuf, FirstChanInBuf, NumOfChansToCopyFromBuf;
            int BufLen = ((int)BufSize / 2);                // length of the buffer in 16 bit elements instead of 8-bit
            Int16[] marshallingArray = new Int16[BufLen];   // array for Int16 copy of buf for marshalling
            UInt16[] ChannelCounts = new UInt16[BufLen];    // buffer for final UInt16 count data

            // Copy pBuf into an Int16 array then convert to UInt16 when used: 
            // This is required due to CLR restraints on IntPtr and Marshal.Copy() for unmanaged code array pointers
            if (BufLen != 0) // if theres data to copy ie not end
            {
                Marshal.Copy(pBuf, marshallingArray, 0, (marshallingArray.Length)); // copy

                TotalAmountOfDataReceived += BufLen;  // By default, C# arithmetic is done in an unchecked context, meaning values will roll over. So it will eventually rollover reasonably at int.MaxValue (+1 %16)
                NumOfChansToCopyFromBuf = Math.Min(16, BufLen); // Note: Just getting 16 for display here
                LastChanInBuf = (TotalAmountOfDataReceived + 15) % 16;   // channel # of actual last entry in this buf of data (ie 15)
                FirstChanInBuf = (16 + (LastChanInBuf - NumOfChansToCopyFromBuf + 1)) % 16;  // zero based (ie 0)

                string data="";
                for (int i = 0; i < NumOfChansToCopyFromBuf; i++)
                {
                    ChannelCounts[(FirstChanInBuf + i) % 16] = (UInt16)marshallingArray[BufLen - NumOfChansToCopyFromBuf + i];
                    data += "ch" + i.ToString() + " " + ChannelCounts[(FirstChanInBuf + i) % 16].ToString() + "\r\n";
                }
                data += "\r\n";

                DataTextBox.BeginInvoke(new UpdateDataFunc(UpdateData), data);
            }

            // Because the display can be slow, and this is called from a worker
            // thread, we just load each channel's last reading for display:
            // Any direct processing of the data, like a Fourier transform etc, would go here instead.
            if (BufLen >= 16) // if enough data to display not end
            {
                // Must be run asynchronously on the UI thread (BeginInvoke vs. Invoke) (members of the ISynchronizeInvoke interface)
                // ...asynchronous version of Invoke, which returns immediately and arranges for the method to run on the UI thread at some point in the future
                // Similar to but not the same as Asyncronous delegate Invocation to prevent thread blocking of the UI thread

                //  Display all the new data through a single delegate Invoke on one control:
                //VoltLabel[0].BeginInvoke(new UpdateGUIFormDel(UpdateGUIFormDisplay), new object[] { ChannelCounts });
            }

        }
        
        private void Form1_Load(object sender, EventArgs e)
        {
            myCallBack = new AIOUSB.ADCallback(ADCallbackReport);
        }

        private void StopButton_Click(object sender, EventArgs e)
        {
            double Hz = 0.0;
            var status = AIOUSB.CTR_StartOutputFreq(DeviceIndex, 0, ref Hz); // stop any running ADC counter
            StatusTextBox.Text += "stopped ADC clock: code " + status.ToString() + "\r\n";
            status = AIOUSB.AIOUSB_CloseDevice(DeviceIndex);
            StatusTextBox.Text += "closed card: code " + status.ToString() + "\r\n";


        }
    }
}
